@@ -35,7 +35,7 @@
 #include "gstv4l2videoenc.h"
 
 #include <string.h>
-#include <glib/gi18n-lib.h>
+#include <gst/gst-i18n-plugin.h>
 
 GST_DEBUG_CATEGORY_STATIC (gst_v4l2_video_enc_debug);
 #define GST_CAT_DEFAULT gst_v4l2_video_enc_debug
@@ -45,7 +45,6 @@ typedef struct
   gchar *device;
   GstCaps *sink_caps;
   GstCaps *src_caps;
-  const GstV4l2Codec *codec;
 } GstV4l2VideoEncCData;
 
 enum
@@ -110,12 +109,11 @@ static gboolean
 gst_v4l2_video_enc_open (GstVideoEncoder * encoder)
 {
   GstV4l2VideoEnc *self = GST_V4L2_VIDEO_ENC (encoder);
-  GstV4l2Error error = GST_V4L2_ERROR_INIT;
-  GstCaps *tmp_caps, *codec_caps;
+  GstCaps *codec_caps;
 
   GST_DEBUG_OBJECT (self, "Opening");
 
-  if (!gst_v4l2_object_open (self->v4l2output, &error))
+  if (!gst_v4l2_object_open (self->v4l2output))
     goto failure;
 
   if (!gst_v4l2_object_open_shared (self->v4l2capture, self->v4l2output))
@@ -124,15 +122,6 @@ gst_v4l2_video_enc_open (GstVideoEncoder * encoder)
   self->probed_sinkcaps = gst_v4l2_object_probe_caps (self->v4l2output,
       gst_v4l2_object_get_raw_caps ());
 
-  /*
-   * Relax the framerate to always allow 0/1 regardless of what the driver
-   * wants. This improve compatibility of the encoder with sources which may
-   * not know the frame rate
-   */
-  tmp_caps = gst_caps_copy (self->probed_sinkcaps);
-  gst_caps_set_simple (tmp_caps, "framerate", GST_TYPE_FRACTION, 0, 1, NULL);
-  gst_caps_append (self->probed_sinkcaps, tmp_caps);
-
   if (gst_caps_is_empty (self->probed_sinkcaps))
     goto no_raw_format;
 
@@ -169,8 +158,6 @@ failure:
   gst_caps_replace (&self->probed_srccaps, NULL);
   gst_caps_replace (&self->probed_sinkcaps, NULL);
 
-  gst_v4l2_error (self, &error);
-
   return FALSE;
 }
 
@@ -222,6 +209,7 @@ gst_v4l2_video_enc_stop (GstVideoEncoder * encoder)
 
   /* Should have been flushed already */
   g_assert (g_atomic_int_get (&self->active) == FALSE);
+  g_assert (g_atomic_int_get (&self->processing) == FALSE);
 
   gst_v4l2_object_stop (self->v4l2output);
   gst_v4l2_object_stop (self->v4l2capture);
@@ -295,7 +283,7 @@ gst_v4l2_video_enc_finish (GstVideoEncoder * encoder)
   }
 
   /* and ensure the processing thread has stopped in case another error
-   * occurred. */
+   * occured. */
   gst_v4l2_object_unlock (self->v4l2capture);
   gst_pad_stop_task (encoder->srcpad);
   GST_VIDEO_ENCODER_STREAM_LOCK (encoder);
@@ -309,21 +297,6 @@ done:
   return ret;
 }
 
-static gboolean
-gst_v4l2_video_enc_flush (GstVideoEncoder * encoder)
-{
-  GstV4l2VideoEnc *self = GST_V4L2_VIDEO_ENC (encoder);
-
-  GST_DEBUG_OBJECT (self, "Flushing");
-
-  self->output_flow = GST_FLOW_OK;
-
-  gst_v4l2_object_unlock_stop (self->v4l2output);
-  gst_v4l2_object_unlock_stop (self->v4l2capture);
-
-  return TRUE;
-}
-
 static gboolean
 gst_v4l2_video_enc_set_format (GstVideoEncoder * encoder,
     GstVideoCodecState * state)
@@ -342,8 +315,8 @@ gst_v4l2_video_enc_set_format (GstVideoEncoder * encoder,
       return TRUE;
     }
 
-    gst_v4l2_video_enc_finish (encoder);
-    gst_v4l2_video_enc_flush (encoder);
+    if (gst_v4l2_video_enc_finish (encoder) != GST_FLOW_OK)
+      return FALSE;
 
     gst_v4l2_object_stop (self->v4l2output);
     gst_v4l2_object_stop (self->v4l2capture);
@@ -365,9 +338,6 @@ gst_v4l2_video_enc_set_format (GstVideoEncoder * encoder,
     return FALSE;
   }
 
-  /* best effort */
-  gst_v4l2_object_setup_padding (self->v4l2output);
-
   self->input_state = gst_video_codec_state_ref (state);
 
   GST_DEBUG_OBJECT (self, "output caps: %" GST_PTR_FORMAT, state->caps);
@@ -375,6 +345,34 @@ gst_v4l2_video_enc_set_format (GstVideoEncoder * encoder,
   return ret;
 }
 
+static gboolean
+gst_v4l2_video_enc_flush (GstVideoEncoder * encoder)
+{
+  GstV4l2VideoEnc *self = GST_V4L2_VIDEO_ENC (encoder);
+
+  GST_DEBUG_OBJECT (self, "Flushing");
+
+  /* Ensure the processing thread has stopped for the reverse playback
+   * iscount case */
+  if (g_atomic_int_get (&self->processing)) {
+    GST_VIDEO_ENCODER_STREAM_UNLOCK (encoder);
+
+    gst_v4l2_object_unlock_stop (self->v4l2output);
+    gst_v4l2_object_unlock_stop (self->v4l2capture);
+    gst_pad_stop_task (encoder->srcpad);
+
+    GST_VIDEO_ENCODER_STREAM_UNLOCK (encoder);
+
+  }
+
+  self->output_flow = GST_FLOW_OK;
+
+  gst_v4l2_object_unlock_stop (self->v4l2output);
+  gst_v4l2_object_unlock_stop (self->v4l2capture);
+
+  return TRUE;
+}
+
 struct ProfileLevelCtx
 {
   GstV4l2VideoEnc *self;
@@ -421,9 +419,8 @@ negotiate_profile_and_level (GstCapsFeatures * features, GstStructure * s,
   GQueue profiles = G_QUEUE_INIT;
   GQueue levels = G_QUEUE_INIT;
   gboolean failed = FALSE;
-  const GstV4l2Codec *codec = klass->codec;
 
-  if (codec->profile_cid && get_string_list (s, "profile", &profiles)) {
+  if (klass->profile_cid && get_string_list (s, "profile", &profiles)) {
     GList *l;
 
     for (l = profiles.head; l; l = l->next) {
@@ -433,9 +430,8 @@ negotiate_profile_and_level (GstCapsFeatures * features, GstStructure * s,
 
       GST_TRACE_OBJECT (ctx->self, "Trying profile %s", profile);
 
-      control.id = codec->profile_cid;
-
-      control.value = v4l2_profile = codec->profile_from_string (profile);
+      control.id = klass->profile_cid;
+      control.value = v4l2_profile = klass->profile_from_string (profile);
 
       if (control.value < 0)
         continue;
@@ -446,7 +442,7 @@ negotiate_profile_and_level (GstCapsFeatures * features, GstStructure * s,
         break;
       }
 
-      profile = codec->profile_to_string (control.value);
+      profile = klass->profile_to_string (control.value);
 
       if (control.value == v4l2_profile) {
         ctx->profile = profile;
@@ -466,7 +462,7 @@ negotiate_profile_and_level (GstCapsFeatures * features, GstStructure * s,
     g_queue_clear (&profiles);
   }
 
-  if (!failed && codec->level_cid && get_string_list (s, "level", &levels)) {
+  if (!failed && klass->level_cid && get_string_list (s, "level", &levels)) {
     GList *l;
 
     for (l = levels.head; l; l = l->next) {
@@ -476,8 +472,8 @@ negotiate_profile_and_level (GstCapsFeatures * features, GstStructure * s,
 
       GST_TRACE_OBJECT (ctx->self, "Trying level %s", level);
 
-      control.id = codec->level_cid;
-      control.value = v4l2_level = codec->level_from_string (level);
+      control.id = klass->level_cid;
+      control.value = v4l2_level = klass->level_from_string (level);
 
       if (control.value < 0)
         continue;
@@ -488,7 +484,7 @@ negotiate_profile_and_level (GstCapsFeatures * features, GstStructure * s,
         break;
       }
 
-      level = codec->level_to_string (control.value);
+      level = klass->level_to_string (control.value);
 
       if (control.value == v4l2_level) {
         ctx->level = level;
@@ -512,26 +508,6 @@ negotiate_profile_and_level (GstCapsFeatures * features, GstStructure * s,
   return failed;
 }
 
-static GstCaps *
-gst_v4l2_video_enc_sink_getcaps (GstVideoEncoder * encoder, GstCaps * filter)
-{
-  GstV4l2VideoEnc *self = GST_V4L2_VIDEO_ENC (encoder);
-  GstCaps *probed_caps = NULL;
-  GstCaps *caps;
-
-  if (self->probed_sinkcaps)
-    probed_caps = gst_caps_ref (self->probed_sinkcaps);
-
-  caps = gst_video_encoder_proxy_getcaps (encoder, probed_caps, filter);
-
-  if (probed_caps)
-    gst_caps_unref (probed_caps);
-
-  GST_DEBUG_OBJECT (self, "Returning sink caps %" GST_PTR_FORMAT, caps);
-
-  return caps;
-}
-
 static gboolean
 gst_v4l2_video_enc_negotiate (GstVideoEncoder * encoder)
 {
@@ -542,7 +518,6 @@ gst_v4l2_video_enc_negotiate (GstVideoEncoder * encoder)
   struct ProfileLevelCtx ctx = { self, NULL, NULL };
   GstVideoCodecState *state;
   GstStructure *s;
-  const GstV4l2Codec *codec = klass->codec;
 
   GST_DEBUG_OBJECT (self, "Negotiating %s profile and level.",
       klass->codec_name);
@@ -551,9 +526,6 @@ gst_v4l2_video_enc_negotiate (GstVideoEncoder * encoder)
   if (self->input_state)
     return TRUE;
 
-  if (!codec)
-    goto done;
-
   allowed_caps = gst_pad_get_allowed_caps (GST_VIDEO_ENCODER_SRC_PAD (encoder));
 
   if (allowed_caps) {
@@ -569,31 +541,28 @@ gst_v4l2_video_enc_negotiate (GstVideoEncoder * encoder)
     if (gst_caps_foreach (allowed_caps, negotiate_profile_and_level, &ctx)) {
       goto no_profile_level;
     }
-
-    gst_caps_unref (allowed_caps);
-    allowed_caps = NULL;
   }
 
-  if (codec->profile_cid && !ctx.profile) {
+  if (klass->profile_cid && !ctx.profile) {
     struct v4l2_control control = { 0, };
 
-    control.id = codec->profile_cid;
+    control.id = klass->profile_cid;
 
     if (v4l2object->ioctl (v4l2object->video_fd, VIDIOC_G_CTRL, &control) < 0)
       goto g_ctrl_failed;
 
-    ctx.profile = codec->profile_to_string (control.value);
+    ctx.profile = klass->profile_to_string (control.value);
   }
 
-  if (codec->level_cid && !ctx.level) {
+  if (klass->level_cid && !ctx.level) {
     struct v4l2_control control = { 0, };
 
-    control.id = codec->level_cid;
+    control.id = klass->level_cid;
 
     if (v4l2object->ioctl (v4l2object->video_fd, VIDIOC_G_CTRL, &control) < 0)
       goto g_ctrl_failed;
 
-    ctx.level = codec->level_to_string (control.value);
+    ctx.level = klass->level_to_string (control.value);
   }
 
   GST_DEBUG_OBJECT (self, "Selected %s profile %s at level %s",
@@ -602,16 +571,12 @@ gst_v4l2_video_enc_negotiate (GstVideoEncoder * encoder)
   state = gst_video_encoder_get_output_state (encoder);
   s = gst_caps_get_structure (state->caps, 0);
 
-  if (codec->profile_cid)
+  if (klass->profile_cid)
     gst_structure_set (s, "profile", G_TYPE_STRING, ctx.profile, NULL);
 
-  if (codec->level_cid)
+  if (klass->level_cid)
     gst_structure_set (s, "level", G_TYPE_STRING, ctx.level, NULL);
 
-  gst_video_codec_state_unref (state);
-  state = NULL;
-
-done:
   if (!GST_VIDEO_ENCODER_CLASS (parent_class)->negotiate (encoder))
     return FALSE;
 
@@ -633,77 +598,69 @@ not_negotiated:
   return FALSE;
 }
 
-static gboolean
-check_system_frame_number_too_old (guint32 current, guint32 old)
+static GstVideoCodecFrame *
+gst_v4l2_video_enc_get_oldest_frame (GstVideoEncoder * encoder)
 {
-  guint32 absdiff = current > old ? current - old : old - current;
+  GstVideoCodecFrame *frame = NULL;
+  GList *frames, *l;
+  gint count = 0;
 
-  /* More than 100 frames in the past, or current wrapped around */
-  if (absdiff > 100) {
-    /* Wraparound and difference is actually smaller than 100 */
-    if (absdiff > G_MAXUINT32 - 100)
-      return FALSE;
-    return TRUE;
+  frames = gst_video_encoder_get_frames (encoder);
+
+  for (l = frames; l != NULL; l = l->next) {
+    GstVideoCodecFrame *f = l->data;
+
+    if (!frame || frame->pts > f->pts)
+      frame = f;
+
+    count++;
   }
 
-  return FALSE;
+  if (frame) {
+    GST_LOG_OBJECT (encoder,
+        "Oldest frame is %d %" GST_TIME_FORMAT
+        " and %d frames left",
+        frame->system_frame_number, GST_TIME_ARGS (frame->pts), count - 1);
+    gst_video_codec_frame_ref (frame);
+  }
+
+  g_list_free_full (frames, (GDestroyNotify) gst_video_codec_frame_unref);
+
+  return frame;
 }
 
 static void
 gst_v4l2_video_enc_loop (GstVideoEncoder * encoder)
 {
   GstV4l2VideoEnc *self = GST_V4L2_VIDEO_ENC (encoder);
-  GstBufferPool *pool = gst_v4l2_object_get_buffer_pool (self->v4l2capture);
-  GstV4l2BufferPool *cpool = GST_V4L2_BUFFER_POOL (pool);
   GstVideoCodecFrame *frame;
   GstBuffer *buffer = NULL;
   GstFlowReturn ret;
 
   GST_LOG_OBJECT (encoder, "Allocate output buffer");
 
-  ret = gst_buffer_pool_acquire_buffer (pool, &buffer, NULL);
-  if (ret != GST_FLOW_OK) {
-    gst_object_unref (cpool);
+  buffer = gst_video_encoder_allocate_output_buffer (encoder,
+      self->v4l2capture->info.size);
+
+  if (NULL == buffer) {
+    ret = GST_FLOW_FLUSHING;
     goto beach;
   }
 
+
   /* FIXME Check if buffer isn't the last one here */
 
   GST_LOG_OBJECT (encoder, "Process output buffer");
-  ret = gst_v4l2_buffer_pool_process (cpool, &buffer, NULL);
-  gst_object_unref (cpool);
+  ret =
+      gst_v4l2_buffer_pool_process (GST_V4L2_BUFFER_POOL
+      (self->v4l2capture->pool), &buffer);
+
   if (ret != GST_FLOW_OK)
     goto beach;
 
-  if (GST_BUFFER_TIMESTAMP (buffer) % GST_SECOND != 0)
-    GST_ERROR_OBJECT (encoder,
-        "Driver bug detected - check driver with v4l2-compliance from http://git.linuxtv.org/v4l-utils.git");
-  GST_LOG_OBJECT (encoder, "Got buffer for frame number %u",
-      (guint32) (GST_BUFFER_PTS (buffer) / GST_SECOND));
-  frame =
-      gst_video_encoder_get_frame (encoder,
-      GST_BUFFER_TIMESTAMP (buffer) / GST_SECOND);
+  frame = gst_v4l2_video_enc_get_oldest_frame (encoder);
 
   if (frame) {
-    GstVideoCodecFrame *oldest_frame;
-    gboolean warned = FALSE;
-
-    /* Garbage collect old frames in case of codec bugs */
-    while ((oldest_frame = gst_video_encoder_get_oldest_frame (encoder)) &&
-        check_system_frame_number_too_old (frame->system_frame_number,
-            oldest_frame->system_frame_number)) {
-      gst_video_encoder_finish_frame (encoder, oldest_frame);
-      oldest_frame = NULL;
-
-      if (!warned) {
-        g_warning ("%s: Too old frames, bug in encoder -- please file a bug",
-            GST_ELEMENT_NAME (encoder));
-        warned = TRUE;
-      }
-    }
-    if (oldest_frame)
-      gst_video_codec_frame_unref (oldest_frame);
-
     /* At this point, the delta unit buffer flag is already correctly set by
      * gst_v4l2_buffer_pool_process. Since gst_video_encoder_finish_frame
      * will overwrite it from GST_VIDEO_CODEC_FRAME_IS_SYNC_POINT (frame),
@@ -731,6 +688,7 @@ beach:
 
   gst_buffer_replace (&buffer, NULL);
   self->output_flow = ret;
+  g_atomic_int_set (&self->processing, FALSE);
   gst_v4l2_object_unlock (self->v4l2output);
   gst_pad_pause_task (encoder->srcpad);
 }
@@ -738,6 +696,12 @@ beach:
 static void
 gst_v4l2_video_enc_loop_stopped (GstV4l2VideoEnc * self)
 {
+  if (g_atomic_int_get (&self->processing)) {
+    GST_DEBUG_OBJECT (self, "Early stop of encoding thread");
+    self->output_flow = GST_FLOW_FLUSHING;
+    g_atomic_int_set (&self->processing, FALSE);
+  }
+
   GST_DEBUG_OBJECT (self, "Encoding task destroyed: %s",
       gst_flow_get_name (self->output_flow));
 
@@ -750,73 +714,42 @@ gst_v4l2_video_enc_handle_frame (GstVideoEncoder * encoder,
   GstV4l2VideoEnc *self = GST_V4L2_VIDEO_ENC (encoder);
   GstFlowReturn ret = GST_FLOW_OK;
   GstTaskState task_state;
-  gboolean active;
 
-  GST_LOG_OBJECT (self, "Handling frame %d", frame->system_frame_number);
+  GST_DEBUG_OBJECT (self, "Handling frame %d", frame->system_frame_number);
 
   if (G_UNLIKELY (!g_atomic_int_get (&self->active)))
     goto flushing;
 
-  task_state = gst_pad_get_task_state (GST_VIDEO_ENCODER_SRC_PAD (self));
+  task_state = gst_pad_get_task_state (GST_VIDEO_DECODER_SRC_PAD (self));
   if (task_state == GST_TASK_STOPPED || task_state == GST_TASK_PAUSED) {
-    /* It is possible that the processing thread stopped due to an error or
-     * when the last buffer has been met during the draining process. */
+    GstBufferPool *pool = GST_BUFFER_POOL (self->v4l2output->pool);
+
+    /* It possible that the processing thread stopped due to an error */
     if (self->output_flow != GST_FLOW_OK &&
-        self->output_flow != GST_FLOW_FLUSHING &&
-        self->output_flow != GST_V4L2_FLOW_LAST_BUFFER) {
-      GST_DEBUG_OBJECT (self, "Processing loop stopped with error: %s, leaving",
-          gst_flow_get_name (self->output_flow));
+        self->output_flow != GST_FLOW_FLUSHING) {
+      GST_DEBUG_OBJECT (self, "Processing loop stopped with error, leaving");
       ret = self->output_flow;
       goto drop;
     }
-  }
 
-  {
-    /* Ensure input internal output pool is active */
-    GstBufferPool *opool = gst_v4l2_object_get_buffer_pool (self->v4l2output);
-    if (!gst_buffer_pool_is_active (opool)) {
-      GstStructure *config = gst_buffer_pool_get_config (opool);
-      guint min = MAX (self->v4l2output->min_buffers,
-          GST_V4L2_MIN_BUFFERS (self->v4l2output));
+    /* Ensure input internal pool is active */
+    if (!gst_buffer_pool_is_active (pool)) {
+      GstStructure *config = gst_buffer_pool_get_config (pool);
+      guint min = MAX (self->v4l2output->min_buffers, GST_V4L2_MIN_BUFFERS);
 
       gst_buffer_pool_config_set_params (config, self->input_state->caps,
-          self->v4l2output->info.vinfo.size, min, min);
+          self->v4l2output->info.size, min, min);
 
       /* There is no reason to refuse this config */
-      if (!gst_buffer_pool_set_config (opool, config)) {
-        config = gst_buffer_pool_get_config (opool);
+      if (!gst_buffer_pool_set_config (pool, config))
+        goto activate_failed;
 
-        if (!gst_buffer_pool_config_validate_params (config,
-                self->input_state->caps, self->v4l2output->info.vinfo.size, min,
-                min)) {
-          gst_structure_free (config);
-          gst_object_unref (opool);
-          goto activate_failed;
-        }
-
-        if (!gst_buffer_pool_set_config (opool, config)) {
-          gst_object_unref (opool);
-          goto activate_failed;
-        }
-      }
-
-      if (!gst_buffer_pool_set_active (opool, TRUE)) {
-        gst_object_unref (opool);
+      if (!gst_buffer_pool_set_active (pool, TRUE))
         goto activate_failed;
-      }
     }
 
-    gst_object_unref (opool);
-  }
-
-  if (task_state == GST_TASK_STOPPED || task_state == GST_TASK_PAUSED) {
-    {
-      GstBufferPool *cpool =
-          gst_v4l2_object_get_buffer_pool (self->v4l2capture);
-      active = gst_buffer_pool_set_active (cpool, TRUE);
-      gst_object_unref (cpool);
-    }
-    if (!active) {
+    if (!gst_buffer_pool_set_active
+        (GST_BUFFER_POOL (self->v4l2capture->pool), TRUE)) {
       GST_WARNING_OBJECT (self, "Could not activate capture buffer pool.");
       goto activate_failed;
     }
@@ -832,32 +765,15 @@ gst_v4l2_video_enc_handle_frame (GstVideoEncoder * encoder,
   }
 
   if (frame->input_buffer) {
-    /* Process force keyframe event if it was passed */
-    if (GST_VIDEO_CODEC_FRAME_IS_FORCE_KEYFRAME (frame)) {
-      struct v4l2_control ctrl = { V4L2_CID_MPEG_VIDEO_FORCE_KEY_FRAME, 1 };
-      if (self->v4l2output->ioctl (self->v4l2output->video_fd, VIDIOC_S_CTRL,
-              &ctrl) < 0)
-        GST_ELEMENT_WARNING (self, RESOURCE, FAILED,
-            (_("Failed to force keyframe.")),
-            ("VIDIOC_S_CTRL (V4L2_CID_MPEG_VIDEO_FORCE_KEY_FRAME) failed: %s (%d)",
-                g_strerror (errno), errno));
-    }
-
     GST_VIDEO_ENCODER_STREAM_UNLOCK (encoder);
-    GST_LOG_OBJECT (encoder, "Passing buffer with frame number %u",
-        frame->system_frame_number);
-
-    {
-      GstBufferPool *opool = gst_v4l2_object_get_buffer_pool (self->v4l2output);
-      ret = gst_v4l2_buffer_pool_process (GST_V4L2_BUFFER_POOL (opool),
-          &frame->input_buffer, &frame->system_frame_number);
-      gst_object_unref (opool);
-    }
-
+    ret =
+        gst_v4l2_buffer_pool_process (GST_V4L2_BUFFER_POOL
+        (self->v4l2output->pool), &frame->input_buffer);
     GST_VIDEO_ENCODER_STREAM_LOCK (encoder);
 
     if (ret == GST_FLOW_FLUSHING) {
-      if (gst_pad_get_task_state (encoder->srcpad) != GST_TASK_STARTED)
+      if (gst_pad_get_task_state (GST_VIDEO_DECODER_SRC_PAD (self)) !=
+          GST_TASK_STARTED)
         ret = self->output_flow;
       goto drop;
     } else if (ret != GST_FLOW_OK) {
@@ -886,6 +802,7 @@ start_task_failed:
   {
     GST_ELEMENT_ERROR (self, RESOURCE, FAILED,
         (_("Failed to start encoding thread.")), (NULL));
+    g_atomic_int_set (&self->processing, FALSE);
     ret = GST_FLOW_ERROR;
     goto drop;
   }
@@ -929,25 +846,18 @@ gst_v4l2_video_enc_decide_allocation (GstVideoEncoder *
   }
   gst_caps_unref (caps);
 
-  /* best effort */
-  gst_v4l2_object_setup_padding (self->v4l2capture);
-
   if (gst_v4l2_object_decide_allocation (self->v4l2capture, query)) {
     GstVideoEncoderClass *enc_class = GST_VIDEO_ENCODER_CLASS (parent_class);
     ret = enc_class->decide_allocation (encoder, query);
   }
 
   /* FIXME This may not be entirely correct, as encoder may keep some
-   * observation without delaying the encoding. Linux Media API need some
+   * observation withouth delaying the encoding. Linux Media API need some
    * more work to explicitly expressed the decoder / encoder latency. This
    * value will then become max latency, and the reported driver latency would
    * become the min latency. */
-  if (!GST_CLOCK_TIME_IS_VALID (self->v4l2capture->duration))
-    self->v4l2capture->duration = gst_util_uint64_scale_int (GST_SECOND, 1, 25);
   latency = self->v4l2capture->min_buffers * self->v4l2capture->duration;
   gst_video_encoder_set_latency (encoder, latency, latency);
-  GST_DEBUG_OBJECT (self, "Setting latency: %" GST_TIME_FORMAT,
-      GST_TIME_ARGS (latency));
 
 done:
   gst_video_codec_state_unref (state);
@@ -1018,14 +928,53 @@ gst_v4l2_video_enc_src_query (GstVideoEncoder * encoder, GstQuery * query)
   return ret;
 }
 
+static gboolean
+gst_v4l2_video_enc_sink_query (GstVideoEncoder * encoder, GstQuery * query)
+{
+  gboolean ret = TRUE;
+  GstV4l2VideoEnc *self = GST_V4L2_VIDEO_ENC (encoder);
+
+  switch (GST_QUERY_TYPE (query)) {
+    case GST_QUERY_CAPS:{
+      GstCaps *filter, *result = NULL;
+      GstPad *pad = GST_VIDEO_ENCODER_SINK_PAD (encoder);
+
+      gst_query_parse_caps (query, &filter);
+
+      if (self->probed_sinkcaps)
+        result = gst_caps_ref (self->probed_sinkcaps);
+      else
+        result = gst_pad_get_pad_template_caps (pad);
+
+      if (filter) {
+        GstCaps *tmp = result;
+        result =
+            gst_caps_intersect_full (filter, tmp, GST_CAPS_INTERSECT_FIRST);
+        gst_caps_unref (tmp);
+      }
+
+      GST_DEBUG_OBJECT (self, "Returning sink caps %" GST_PTR_FORMAT, result);
+
+      gst_query_set_caps_result (query, result);
+      gst_caps_unref (result);
+      break;
+    }
+
+    default:
+      ret = GST_VIDEO_ENCODER_CLASS (parent_class)->sink_query (encoder, query);
+      break;
+  }
+
+  return ret;
+}
+
 static gboolean
 gst_v4l2_video_enc_sink_event (GstVideoEncoder * encoder, GstEvent * event)
 {
   GstV4l2VideoEnc *self = GST_V4L2_VIDEO_ENC (encoder);
   gboolean ret;
-  GstEventType type = GST_EVENT_TYPE (event);
 
-  switch (type) {
+  switch (GST_EVENT_TYPE (event)) {
     case GST_EVENT_FLUSH_START:
       GST_DEBUG_OBJECT (self, "flush start");
       gst_v4l2_object_unlock (self->v4l2output);
@@ -1037,7 +986,7 @@ gst_v4l2_video_enc_sink_event (GstVideoEncoder * encoder, GstEvent * event)
 
   ret = GST_VIDEO_ENCODER_CLASS (parent_class)->sink_event (encoder, event);
 
-  switch (type) {
+  switch (GST_EVENT_TYPE (event)) {
     case GST_EVENT_FLUSH_START:
       gst_pad_stop_task (encoder->srcpad);
       GST_DEBUG_OBJECT (self, "flush start done");
@@ -1143,14 +1092,14 @@ gst_v4l2_video_enc_class_init (GstV4l2VideoEncClass * klass)
   video_encoder_class->flush = GST_DEBUG_FUNCPTR (gst_v4l2_video_enc_flush);
   video_encoder_class->set_format =
       GST_DEBUG_FUNCPTR (gst_v4l2_video_enc_set_format);
-  video_encoder_class->getcaps =
-      GST_DEBUG_FUNCPTR (gst_v4l2_video_enc_sink_getcaps);
   video_encoder_class->negotiate =
       GST_DEBUG_FUNCPTR (gst_v4l2_video_enc_negotiate);
   video_encoder_class->decide_allocation =
       GST_DEBUG_FUNCPTR (gst_v4l2_video_enc_decide_allocation);
   video_encoder_class->propose_allocation =
       GST_DEBUG_FUNCPTR (gst_v4l2_video_enc_propose_allocation);
+  video_encoder_class->sink_query =
+      GST_DEBUG_FUNCPTR (gst_v4l2_video_enc_sink_query);
   video_encoder_class->src_query =
       GST_DEBUG_FUNCPTR (gst_v4l2_video_enc_src_query);
   video_encoder_class->sink_event =
@@ -1172,8 +1121,8 @@ gst_v4l2_video_enc_subclass_init (gpointer g_class, gpointer data)
   GstV4l2VideoEncCData *cdata = data;
 
   klass->default_device = cdata->device;
-  klass->codec = cdata->codec;
 
+  /* Note: gst_pad_template_new() take the floating ref from the caps */
   gst_element_class_add_pad_template (element_class,
       gst_pad_template_new ("sink", GST_PAD_SINK, GST_PAD_ALWAYS,
           cdata->sink_caps));
@@ -1210,9 +1159,8 @@ gst_v4l2_is_video_enc (GstCaps * sink_caps, GstCaps * src_caps,
 
 void
 gst_v4l2_video_enc_register (GstPlugin * plugin, GType type,
-    const char *codec_name, const gchar * basename, const gchar * device_path,
-    const GstV4l2Codec * codec, gint video_fd, GstCaps * sink_caps,
-    GstCaps * codec_caps, GstCaps * src_caps)
+    const char *codec, const gchar * basename, const gchar * device_path,
+    GstCaps * sink_caps, GstCaps * codec_caps, GstCaps * src_caps)
 {
   GstCaps *filtered_caps;
   GTypeQuery type_query;
@@ -1220,28 +1168,13 @@ gst_v4l2_video_enc_register (GstPlugin * plugin, GType type,
   GType subtype;
   gchar *type_name;
   GstV4l2VideoEncCData *cdata;
-  GValue value = G_VALUE_INIT;
 
   filtered_caps = gst_caps_intersect (src_caps, codec_caps);
-  GST_MINI_OBJECT_FLAG_SET (filtered_caps, GST_MINI_OBJECT_FLAG_MAY_BE_LEAKED);
-
-  if (codec != NULL && video_fd != -1) {
-    if (gst_v4l2_codec_probe_levels (codec, video_fd, &value)) {
-      gst_caps_set_value (filtered_caps, "level", &value);
-      g_value_unset (&value);
-    }
-
-    if (gst_v4l2_codec_probe_profiles (codec, video_fd, &value)) {
-      gst_caps_set_value (filtered_caps, "profile", &value);
-      g_value_unset (&value);
-    }
-  }
 
   cdata = g_new0 (GstV4l2VideoEncCData, 1);
   cdata->device = g_strdup (device_path);
   cdata->sink_caps = gst_caps_ref (sink_caps);
-  cdata->src_caps = filtered_caps;
-  cdata->codec = codec;
+  cdata->src_caps = gst_caps_ref (filtered_caps);
 
   g_type_query (type, &type_query);
   memset (&type_info, 0, sizeof (type_info));
@@ -1255,11 +1188,11 @@ gst_v4l2_video_enc_register (GstPlugin * plugin, GType type,
    * v4l2h264enc, for any additional encoders, we create unique names. Encoder
    * names may change between boots, so this should help gain stable names for
    * the most common use cases. */
-  type_name = g_strdup_printf ("v4l2%senc", codec_name);
+  type_name = g_strdup_printf ("v4l2%senc", codec);
 
   if (g_type_from_name (type_name) != 0) {
     g_free (type_name);
-    type_name = g_strdup_printf ("v4l2%s%senc", basename, codec_name);
+    type_name = g_strdup_printf ("v4l2%s%senc", basename, codec);
   }
 
   subtype = g_type_register_static (type, type_name, &type_info, 0);
